@@ -1,4 +1,4 @@
-# Copyright 2015-2016 F-Secure
+# Copyright 2015-2017 F-Secure
 
 # Licensed under the Apache License, Version 2.0 (the "License"); you
 # may not use this file except in compliance with the License.  You may
@@ -124,6 +124,7 @@ class SeeContext(Context):
         self._network_mutex = Lock()
         self._mac_address = None
         self._ip4_address = None
+        self._ip6_address = None
 
     def cleanup(self):
         """Claims the resources back."""
@@ -133,11 +134,7 @@ class SeeContext(Context):
     def hypervisor(self):
         """libvirt.virConnect."""
         with self._hypervisor_mutex:
-            if self._resources.hypervisor.isAlive():
-                return self._resources.hypervisor
-            else:
-                raise RuntimeError(self.identifier,
-                                   "Hypervisor connection is closed")
+            return self._resources.hypervisor
 
     @property
     def domain(self):
@@ -149,19 +146,13 @@ class SeeContext(Context):
     def storage_pool(self):
         """libvirt.virStoragePool."""
         with self._storage_pool_mutex:
-            if self._resources.storage_pool.isActive():
-                return self._resources.storage_pool
-            else:
-                raise RuntimeError(self.identifier, "Storage Pool unavailable.")
+            return self._resources.storage_pool
 
     @property
     def network(self):
         """libvirt.virNetwork."""
         with self._network_mutex:
-            if self._resources.network.isActive():
-                return self._resources.network
-            else:
-                raise RuntimeError(self.identifier, "Network is unavailable.")
+            return self._resources.network
 
     @property
     def mac_address(self):
@@ -191,18 +182,39 @@ class SeeContext(Context):
 
         """
         if self._ip4_address is None and self.network is not None:
-            self._ip4_address = self._get_ip_address()
+            self._ip4_address = self._get_ip_address(
+                libvirt.VIR_IP_ADDR_TYPE_IPV4)
 
         return self._ip4_address
 
-    def _get_ip_address(self):
+    @property
+    def ip6_address(self):
+        """Returns the IPv6 address of the network interface.
+
+        If multiple interfaces are provided,
+        the address of the first found is returned.
+
+        """
+        if self._ip6_address is None and self.network is not None:
+            self._ip6_address = self._get_ip_address(
+                libvirt.VIR_IP_ADDR_TYPE_IPV6)
+
+        return self._ip6_address
+
+    def _get_ip_address(self, address_type):
         mac = self.mac_address
 
-        for lease in self.network.DHCPLeases():
-            if mac == lease.get('mac'):
-                return lease.get('ipaddr')
+        try:
+            interfaces = self.domain.interfaceAddresses(
+                libvirt.VIR_DOMAIN_INTERFACE_ADDRESSES_SRC_LEASE)
+        except AttributeError:  # libvirt < 1.3.0
+            pass
+        else:
+            return interface_lookup(interfaces, mac, address_type)
 
-        return None
+        for lease in self.network.DHCPLeases():
+            if mac == lease.get('mac') and lease.get('type') == address_type:
+                return lease.get('ipaddr')
 
     def poweron(self, **kwargs):
         """
@@ -338,3 +350,12 @@ class SeeContext(Context):
             command(*args)
         except libvirt.libvirtError as error:
             raise RuntimeError("Unable to execute command. %s" % error)
+
+
+def interface_lookup(interfaces, hwaddr, address_type):
+    """Search the address within the interface list."""
+    for interface in interfaces.values():
+        if interface.get('hwaddr') == hwaddr:
+            for address in interface.get('addrs'):
+                if address.get('type') == address_type:
+                    return address.get('addr')
